#include "main.h"
//------------------------------------------------
static const char *TAG = "main";
//------------------------------------------------
typedef struct
{
  unsigned char y_pos;
  unsigned char x_pos;
  char *str;
} qLCDData;
//------------------------------------------------
xQueueHandle lcd_string_queue = NULL;
//------------------------------------------------
void vLCDTask(void* arg)
{
  BaseType_t xStatus;
  qLCDData xReceivedData;
  for(;;) {
    xStatus = xQueueReceive(lcd_string_queue, &xReceivedData, 10000 /portTICK_RATE_MS);
    if (xStatus == pdPASS)
    {
      LCD_SetPos(xReceivedData.x_pos,xReceivedData.y_pos);
      LCD_String(xReceivedData.str);
    }
  }
}
//------------------------------------------------
static void periodic_timer_callback(void* arg)
{
  char str1[15] = {0};
  qLCDData xLCDData;
  xLCDData.x_pos = 0;
  xLCDData.y_pos = 0;
  xLCDData.str = str1;
  int64_t time_since_boot = esp_timer_get_time();
  sprintf(str1,"%010lld", time_since_boot);
  xQueueSendToBack(lcd_string_queue, &xLCDData, 0);
}
//------------------------------------------------
static void oneshot_timer_callback(void* arg)
{
  esp_err_t ret;
  char str1[15] = {0};
  qLCDData xLCDData;
  xLCDData.x_pos = 0;
  xLCDData.y_pos = 1;
  xLCDData.str = str1;
  int64_t time_since_boot = esp_timer_get_time();
  sprintf(str1,"%010lld", time_since_boot);
  xQueueSendToBack(lcd_string_queue, &xLCDData, 0);
  esp_timer_handle_t periodic_timer_handle = (esp_timer_handle_t) arg;
  ret = esp_timer_stop(periodic_timer_handle);
  ESP_LOGI(TAG, "esp_timer_stop: %d", ret);
  ret = esp_timer_start_periodic(periodic_timer_handle, 1000000);
  ESP_LOGI(TAG, "esp_timer_start_periodic: %d", ret);
}
//------------------------------------------------
void app_main(void)
{
  esp_err_t ret;
  lcd_string_queue = xQueueCreate(10, sizeof(qLCDData));
  xTaskCreate(vLCDTask, "vLCDTask", 2048, NULL, 2, NULL);
  ret = i2c_ini();
  ESP_LOGI(TAG, "i2c_ini: %d", ret);
  LCD_ini();
  const esp_timer_create_args_t periodic_timer_args = {
          .callback = &periodic_timer_callback,
          .name = "periodic"
  };
  esp_timer_handle_t periodic_timer;
  ret = esp_timer_create(&periodic_timer_args, &periodic_timer);
  ESP_LOGI(TAG, "esp_timer_create: %d", ret);
  ret = esp_timer_start_periodic(periodic_timer, 500000);
  ESP_LOGI(TAG, "esp_timer_start_periodic: %d", ret);
  const esp_timer_create_args_t oneshot_timer_args = {
          .callback = &oneshot_timer_callback,
          .arg = (void*) periodic_timer,
          .name = "one-shot"
  };
  esp_timer_handle_t oneshot_timer;
  ret = esp_timer_create(&oneshot_timer_args, &oneshot_timer);
  ESP_LOGI(TAG, "esp_timer_create: %d", ret);
  vTaskDelay(250 / portTICK_PERIOD_MS);
  ret = esp_timer_start_once(oneshot_timer, 10000000);
  ESP_LOGI(TAG, "esp_timer_start_once: %d", ret);
  while (1) {
    vTaskDelay(100 / portTICK_PERIOD_MS);
  }
}
//------------------------------------------------
